using System;
using System.Collections.Generic;
using System.Text;

namespace DesignPattern
{
    abstract class GUIFactory
    {
        public static GUIFactory GetFactory()
        {
#if OS_TYPE_MAC
            return new MacFactory();
#else
            return new WinFactory(); 
#endif
        }
        public abstract Control CreateButton();
        public abstract Control CreateLabel();
    }

    class WinFactory : GUIFactory
    {
        // Its there just to have the Classdiagram going
        WinButton btn = new WinButton();
        WinLabel lbl = new WinLabel();
        public override Control CreateButton()
        {
            return btn;
        }

        public override Control CreateLabel()
        {
            return lbl;
        }
    }

    class MacFactory : GUIFactory
    {
        // Its there just to have the Classdiagram going
        MacButton btn = new MacButton();
        MacLabel lbl = new MacLabel();
        public override Control CreateButton()
        {
            return btn;
        }

        public override Control CreateLabel()
        {
            return lbl;
        }
    }

    abstract class Control
    {
        public abstract void Show();
    }

    class WinButton : Control
    {
        public override void Show()
        {
            Console.WriteLine("I'm a WinButton: ");
        }
    }

    class MacButton : Control
    {
        public override void Show()
        {
            Console.WriteLine("I'm an MacButton: ");
        }
    }

    class WinLabel : Control
    {
        public override void Show()
        {
            Console.WriteLine("I'm a WinLabel: ");
        }
    }

    class MacLabel : Control
    {
        public override void Show()
        {
            Console.WriteLine("I'm an MacLabel: ");
        }
    }
}
