using System;
using System.Collections;
using System.Collections.Generic;

namespace DesignPattern
{
    public class Graphic
    {
        //Virtual function that prints the graphic.
        public virtual void Print()
        { 
        
        }
    }

    public class CompositeGraphic : Graphic
    {
        //Collection of child graphics.
        List<Graphic> mChildGraphics = new List<Graphic>();

        //Prints the graphic by Delegating to its Child
        public override void Print()
        {
            foreach (Graphic graphic in mChildGraphics)
            {
                graphic.Print();
            }
        }

        //Adds the graphic to the composition.
        public void Add(Graphic graphic)
        {
            mChildGraphics.Add(graphic);
        }

        //Removes the graphic from the composition.
        public void Remove(Graphic graphic)
        {
            mChildGraphics.Remove(graphic);
        }
    }

    public class Ellipse : Graphic
    {

        //Prints the graphic.
        public override void Print()
        {
            Console.WriteLine("Ellipse");
        }

    }
}
