using System;
using System.Collections.Generic;
using System.Text;

namespace DesignPattern
{
    abstract class IReader
    {
        abstract public void Read();
    }

    class FileReader : IReader
    {
        public override void Read()
        {
            Console.WriteLine("File reader - Read");
        }
    }

    class ReaderDecorator : IReader
    {
        IReader reader;
        public ReaderDecorator(IReader rd)
        {
            this.reader = rd;
        }
        public override void Read()
        {
            reader.Read();
        }
    }

    class ZipReader : ReaderDecorator
    {
        public ZipReader(IReader rd)
            : base(rd)
        {
            
        }
        public override void Read()
        {
            Console.WriteLine("ZipReader - Reading");
            base.Read();
        }
    }
    class EncryptedReader : ReaderDecorator
    {
        public EncryptedReader(IReader rd)
            : base(rd)
        {

        }
        public override void Read()
        {
            Console.WriteLine("EncryptedReader - Reading");
            base.Read();
        }
    }
}
