using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace DesignPattern
{
    // "Aggregate"

    abstract class IList
    {
        public abstract Iterator CreateIterator();
    }

    // "ConcreteAggregate"

    class ConcreteList : IList
    {
        private ArrayList items = new ArrayList();

        public override Iterator CreateIterator()
        {
            return new BackwardIterator(this);
        }

        // Property
        public int Count
        {
            get { return items.Count; }
        }

        // Indexer
        public object this[int index]
        {
            get { return items[index]; }
            set { items.Insert(index, value); }
        }
    }

    // "Iterator"

    abstract class Iterator
    {
        public abstract object First();
        public abstract object Next();
    }

    class ForwardIterator : Iterator
    {
        private ConcreteList aggregate;
        private int current = 0;

        // Constructor
        public ForwardIterator(ConcreteList aggregate)
        {
            this.aggregate = aggregate;
        }

        public override object First()
        {
            return aggregate[0];
        }

        public override object Next()
        {
            object ret = null;
            if (current < aggregate.Count - 1)
            {
                ret = aggregate[++current];
            }

            return ret;
        }

   }
    class BackwardIterator : Iterator
    {
        private ConcreteList aggregate;
        private int current = 0;

        // Constructor
        public BackwardIterator(ConcreteList aggregate)
        {
            this.aggregate = aggregate;
            this.current = aggregate.Count - 1;
        }

        public override object First()
        {
            return aggregate[this.aggregate.Count - 1];
        }

        public override object Next()
        {
            object ret = null;
            if (current > 0)
            {
                ret = aggregate[--current];
            }

            return ret;
        }

    }
}
