using System;
using System.Collections.Generic;
using System.Text;

namespace DesignPattern
{
    abstract class Mediator
    {
        public abstract void Send(string message,
          Control colleague);
    }

    class DialogMediator : Mediator
    {
        private ListControl listCtrl;
        private ButtonControl btnCtrl;

        public ListControl ListCtrl
        {
            set { listCtrl = value; }
        }

        public ButtonControl BtnCtrl
        {
            set { btnCtrl = value; }
        }

        public override void Send(string message,
          Control colleague)
        {
            if (colleague == listCtrl)
            {
                btnCtrl.Notify(message);
            }
            else
            {
                listCtrl.Notify(message);
            }
        }
    }

    abstract class Control
    {
        protected Mediator mediator;

        // Constructor
        public Control(Mediator mediator)
        {
            this.mediator = mediator;
        }
    }

    class ListControl : Control
    {
        // Constructor
        public ListControl(Mediator mediator)
            : base(mediator)
        {
        }

        public void Send(string message)
        {
            mediator.Send(message, this);
        }

        public void Notify(string message)
        {
            Console.WriteLine("ListControl gets message: "
              + message);
        }
    }

    class ButtonControl : Control
    {
        // Constructor
        public ButtonControl(Mediator mediator)
            : base(mediator)
        {
        }

        public void Send(string message)
        {
            mediator.Send(message, this);
        }

        public void Notify(string message)
        {
            Console.WriteLine("ButtonControl gets message: "
              + message);
        }
    }
}
