using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;

namespace DesignPattern
{
    abstract public class Subject
    {
        abstract public void Attach(Observer observer);
        abstract public void Detach(Observer observer);
        abstract public void Notify();
    }

    public class Document : Subject
    {

        private List<Observer> observerList = new List<Observer>();

        #region ISubject Members

        public override void Attach(Observer observer)
        {
            observerList.Add(observer);
        }

        public override void Detach(Observer observer)
        {
            observerList.Remove(observer);
        }

        public override void Notify()
        {
            foreach (Observer observer in observerList)
            {
                observer.Update();
            }
        }

        #endregion

    }

    public class XMLDoc : Document
    {
        public object Data
        {
            get { return data; }
            set
            {
                data = value;
                Notify();
            }
        }

        private object data = null;
    }

    abstract public class Observer
    {
        abstract public void Update();
    }

    public class GraphView : Observer
    {
        public GraphView(XMLDoc sub)
        {
            this.sub = sub;
            sub.Attach(this);
        }

        ~GraphView()
        {
            sub.Detach(this);
        }
        public override void Update()
        {
            Console.WriteLine(this.ToString() + "->" + sub.Data.ToString());
        }

        private XMLDoc sub = null;

    }

    public class ChartView : Observer
    {
        public ChartView(XMLDoc sub)
        {
            this.sub = sub;
            sub.Attach(this);
        }

        ~ChartView()
        {
            sub.Detach(this);
        }

        public override void Update()
        {
            Console.WriteLine(this.ToString() + "->" + sub.Data.ToString());
        }

        private XMLDoc sub = null;
    }

}
